import json
import zmq
from zmq.eventloop import ioloop
from zmq.eventloop.zmqstream import ZMQStream

from config import PUBLISHER_PORT, RECEIVER_PORT

ioloop.install()


class EventHandlerBase:
    def __init__(self, anchor, zmq_context=None):
        self.anchor = anchor
        self.zmq_context = zmq_context or zmq.Context.instance()
        self.zmq_sub_socket = self.zmq_context.socket(zmq.SUB)
        self.subscriptions = {self.anchor}
        self.zmq_sub_socket.setsockopt_string(zmq.SUBSCRIBE, self.anchor)
        self.zmq_sub_socket.connect('tcp://localhost:{}'.format(PUBLISHER_PORT))
        self.zmq_sub_stream = ZMQStream(self.zmq_sub_socket)
        self.zmq_push_socket = self.zmq_context.socket(zmq.PUSH)
        self.zmq_push_socket.connect('tcp://localhost:{}'.format(RECEIVER_PORT))
        self.zmq_sub_stream.on_recv(self.event_handler_callback)

    def event_handler_callback(self, msg_parts):
        anchor, message = msg_parts
        data_json = json.loads(message)
        response = self.handle_event(anchor, data_json) if anchor != b'reset' else self.handle_reset(data_json)
        if response is None: return
        encoded_response = json.dumps(response).encode('utf-8')
        self.zmq_push_socket.send_multipart([anchor, encoded_response])

    def handle_event(self, anchor, data_json):
        raise NotImplementedError

    def handle_reset(self, data_json):
        return None

    def message_other(self, anchor, data):
        encoded_anchor = anchor.encode('utf-8')
        message = {
            'anchor': anchor,
            'data': data
        }
        encoded_message = json.dumps(message).encode('utf-8')
        self.zmq_push_socket.send_multipart([encoded_anchor, encoded_message])

    def subscribe(self, anchor):
        if anchor not in self.subscriptions:
            self.subscriptions.add(anchor)
            self.zmq_sub_socket.setsockopt_string(
                zmq.SUBSCRIBE, anchor
            )

    def unsubscribe(self, anchor):
        try:
            self.subscriptions.remove(anchor)
            self.zmq_sub_socket.setsockopt_string(zmq.UNSUBSCRIBE, anchor)
        except KeyError:
            pass

    def unsubscribe_all(self):
        for sub in self.subscriptions:
            self.zmq_sub_socket.setsockopt_string(zmq.UNSUBSCRIBE, sub)
        self.subscriptions.clear()
